#!/usr/bin/bash
# Ci-dessus le shebang ( indique qu'il s'agit d'un script bash )

#****************************************************************************************************#
#****************************-Groupe "Brexit" ( 2020 / 2021 )-***************************************#
#****************************-----Thomas GERVAIS d'ALDIN------***************************************#
#****************************----------Bertisa GREZDA---------***************************************#
#****************************************************************************************************#

# -- Déscriptif du programme-------------------------------------------------------------------------
# -- Script bash traitant des fichiers d'URL afin d'aspirer du contenu textuel de
# pages internet, et effectuer dessus des opérations textométriques de base
# Les résultats de l'execution du script sont accessibles depuis un tableau HTML duquel
# on peut consulter facilement les fichiers crées par le script.
#-----------------------------------------------------------------------------------------------------
# -- Mode d'emploi du programme : 
# 1. Le répertoire courant est PROJET 
# 2. Puis on lance le programme par cette commande :
# bash ./PROGRAMMES/projet.sh ./URLS ./TABLEAUX/tableau.html motif
#-----------------------------------------------------------------------------------------------------
# input du programme : le nom du DOSSIER contenant les fichiers d'URLs est donné en premier argument
# output du programme : le nom du tableau HTML de sortie est donné en second argument
#-----------------------------------------------------------------------------------------------------

# Les arguments du programme stockés dans des variables
dossierURL=$1; # Le chemin vers le repertoire URL
montableau=$2; # Le chemin vers le tableau hmtl à créer
motif=$3; # Le motif cible du programme ( comptage d'occurences , mise en contexte par minigrep ... )

# Suppression des donnees existantes dans nos dossiers
rm ./CONTEXTES/*.txt;
rm ./DUMP-TEXT/*.txt;
rm ./PAGES-ASPIREES/*.txt;

# --------- Début du script --------------

# Ecriture début du fichier HTML de sortie
echo "<html><head><meta charset=\"utf-8\" /></head><body>" > $montableau ;
# Ecriture du motif dans un fichier txt à traiter par minigrep 
echo "MOTIF=$motif" > ./PROGRAMMES/minigrep_multilingue/motif_projet.txt;
# Compteur des fichiers d'URL et des tableaux associés , initialisé à la valeur 1
compteur_tableau=4
# Parcours des fichiers d'URLS contenus dans le dossiers URLS
# Pour chaque fichier d'URLS ...
for fichier in $(ls $dossierURL)
	do
		# Initialisation du compteur d'URLs à la valeur 1 ( pour chaque URL dans chaque fichier txt )
		compteur=1;
		# Indication de l'origine de l'extraction de l'URL sur la sortie standard ( désigne le fichier texte ) 
		echo "URLS extraites de $fichier";
		# Pour chaque fichier d'URL, on va créer un nouveau tableau
		# + Quelques opérations pour définir le style du tableau
		echo "<table align=\"center\" border=\"10px\" bordercolor=\"blue\">" >> $montableau;
		# Création de deux en-têtes du tableau indiquant...
		# ... le titre du tableau ...
		echo "<tr><th colspan=\"12\">TABLEAU $compteur_tableau</th></tr>" >> $montableau;
		# ... et le motif désigné
		echo "<tr><th colspan=\"12\">Motif : $motif</th></tr>">>$montableau;
		# Création des colonnes dans lesquelles nous allons ranger les résultats des traitements pour chaque URL 
		echo "<tr>
						<td>Lien</td> 
						<td>URL</td>
						<td>HTTP_Code</td>
						<td>Page_aspirée</td>
						<td>Dump</td>
						<td>Encodage</td>
						<td>Occurence $motif</td>
						<td>Contextes_txt</td>
						<td>Contextes_html</td>
						<td>Index</td>
						<td>Bigrammes</td>
			   </tr>" >> $montableau;
		# Lecture de chaque ligne du fichier d'URL courant ( 1 ligne = 1 URL )
	while read ligne
			do
				# Affichage sur la sortie standard du traitement de l'URL, avec indication de l'URL en question
				echo "-----------------------------------------------------------";
				echo "TRAITEMENT DE L'URL : $ligne";
				
				# Récupération du http code pour chaque page afin de s'assurer de la disponibilité et de la consultabilité de la page désignée 
				codeHttp=$(curl -o ./PAGES-ASPIREES/"$compteur_tableau-$compteur".html $ligne  -w %{http_code}) ; 
				
				#Test de la valeur du code http: Si le code est '200', les feux sont au vert.
				#   - Si  le code http est 200 , on télécharge et on fait nos opérations
				#   - Sinon on ne fait rien mais on le signale sur la sortie standard
				if [[ $codeHttp == 200 ]]
					then
					# Récupération de la convention d'encodage utilisée pour la page internet courante ...
					encodageURL=$(curl -I $ligne | egrep -i "charset" | cut -f2 -d= | tr [a-z] [A-Z] | tr -d "\r");
					# ... que l'on affiche sur la sortie standard 
					echo "<$encodageURL>";
					# Pour soumettre le texte à nos outils, il faut que la convention d'encodage utilisée soit UTF-8.
					# Le problème c'est que ce n'est pas le cas de toute les pages.
					# Nous allons donc faire une structure conditionelle sur le modèle suivant:
					#	- Si l'encodage est bien UTF-8 , on télécharge et on fait les traitements
					#	- Sinon, on convertit en UTF-8 , puis on télécharge et on fait les traitements
					# Ici , SI l'encodage de la page est bien UTF-8 
					if [[ $encodageURL == "UTF-8" ]]
						then
							# On affiche sur la sortie standard l'encodage de l'URL courante et leurs numéros
							echo "Encodage de l'URL : $encodageURL";
							# Et des informations supplémentaires ( num de tableau , num d'URL , l'URL , le code http et l'encodage utilisé
							echo "<$compteur_tableau><$compteur><$ligne><$codeHttp><$encodageURL>";
							# On telecharge le contenu de la page courante par la commande 'lynx' 
							lynx -dump -nolist -assume_charset="UTF-8" -display_charset="UTF-8" "./PAGES-ASPIREES/$compteur_tableau-$compteur.html" > ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt;
							# On compte les occurences du motif désigné dans la page courante
							compteurMotif=$(egrep -o -i $motif ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | wc -l);
							# On extrait des contextes reduits du motif aux format txt
							egrep -C 2 -i  "$motif" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt > ./CONTEXTES/"$compteur_tableau-$compteur".txt;
							# On extrait des contextes au format html traité par le programme minigrep qui extrait le contexte autour du motif désigné 
							# et nous donne en résultat un fichier du motif dans son contexte, dans un format plus facile à consulter au format html
							perl ./PROGRAMMES/minigrep_multilingue/minigrepmultilingue.pl "UTF-8" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt ./PROGRAMMES/minigrep_multilingue/motif_projet.txt;
							# On sauve le résultat de l'extraction dans le repertoire CONTEXTES 
							mv resultat-extraction.html ./CONTEXTES/"url_$compteur_tableau-$compteur".html;
							# On crée un index des occurences à partir de l'extraction de la page au format texte, que l'on place dans le repertoire CONTEXTES
							egrep -i -o "\w+" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | sort | uniq -c  | sort -r -n -s -k 1,1 > ./DUMP-TEXT/"index_$compteur_tableau-$compteur".txt;
							# On crée toujours à partir de l'extraction de la page au format texte une liste de bigrammes 
							# ( les fichiers 'index2' et 'index3' sont des fichiers de travail pour faire les bigrammes ,
							#   on prendra soin de les supprimer à la fin du script pour éviter de polluer notre environnement de travail )
							tr " " "\n" < ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | tr -s "\n" | egrep -v "^$" > index2.txt;
							tail -n +2 index2.txt > index3.txt;
							paste index2.txt index3.txt | sort | uniq -c | sort -r -n -s -k 1,1 -r > ./DUMP-TEXT/"bigrammes_$compteur_tableau-$compteur".txt;
							# Ecriture dans le tableau de chaque colonne pour chaque ligne dotées de liens cliquables permettant l'accès facile à nos données à partir du tableau crée 
							echo "<tr>
											<td>Lien $compteur</td>
											<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
											<td>$codeHttp</td>
											<td><a href=\"../PAGES-ASPIREES/$compteur_tableau-$compteur.html\">PA $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/UTF-8_$compteur_tableau-$compteur.txt\">DUMP $compteur</a></td>
											<td>$encodageURL</td>
											<td>$compteurMotif</td>
											<td><a href=\"../CONTEXTES/$compteur_tableau-$compteur.txt\"> CONTEXTE TXT $compteur</a></td>
											<td><a href=\"../CONTEXTES/url_$compteur_tableau-$compteur.html\" > CONTEXTE HTML $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/index_$compteur_tableau-$compteur.txt\"> INDEX $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/bigrammes_$compteur_tableau-$compteur.txt\">Bigrammes $compteur</a></td></tr></tr>" >> $montableau;
											
					# SINON ( donc si l'encodage n'est pas UTF-8 )
					else
							# On affiche sur la sortie standard l'encodage de l'URL courante et leurs numéros
							echo "Encodage de l'URL : $encodageURL";
							# Et des informations supplémentaires ( num de tableau , num d'URL , l'URL , le code http et l'encodage utilisé
							echo "<$compteur_tableau><$compteur><$ligne><$codeHttp><$encodageURL>";
							# On telecharge le contenu de la page courante par la commande 'lynx'
							lynx -dump -nolist -assume_charset="$encodageURL" -display_charset="$encodageURL" "./PAGES-ASPIREES/$compteur_tableau-$compteur.html" > ./DUMP-TEXT/pour_transcodage-"$compteur_tableau-$compteur".txt;
							# On transcode par la commande 'iconv'. On a récupéré dans une variable l'encodage source, et on l'utilise pour #l'indiquer à iconv afin de  transcoder vers UTF-8 ( ainsi , on normalise notre corpus )
							iconv -f "$encodageURL" -t UTF-8 ./DUMP-TEXT/pour_transcodage-"$compteur_tableau-$compteur".txt > ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt;
							rm pour_transcodage-*.txt;
							# On compte les occurences du motif désigné dans la page courante
							compteurMotif=$(egrep -o -i $motif ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | wc -l);
							# On extrait des contextes reduits du motif aux format txt
							egrep -C 2 -i  "$motif" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt > ./CONTEXTES/"$compteur_tableau-$compteur".txt;
							# On extrait des contextes au format html traité par le programme minigrep qui extrait le contexte autour du motif désigné 
							# et nous donne en résultat un fichier du motif dans son contexte, dans un format plus facile à consulter au format html
							perl ./PROGRAMMES/minigrep_multilingue/minigrepmultilingue.pl "UTF-8" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt ./PROGRAMMES/minigrep_multilingue/motif_projet.txt;
							# On sauve le résultat de l'extraction dans le repertoire CONTEXTES
							mv resultat-extraction.html ./CONTEXTES/"url_$compteur_tableau-$compteur".html;
							# On crée un index des occurences à partir de l'extraction de la page au format texte, que l'on place dans le repertoire CONTEXTES
							egrep -i -o "\w+" ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | sort | uniq -c  | sort -r -n -s -k 1,1 > ./DUMP-TEXT/"index_$compteur_tableau-$compteur".txt;
							# On crée toujours à partir de l'extraction de la page au format texte une liste de bigrammes 
							# ( les fichiers 'index2' et 'index3' sont des fichiers de travail pour faire les bigrammes ,
							#   on prendra soin de les supprimer à la fin du script pour éviter de polluer notre environnement de travail )
							tr " " "\n" < ./DUMP-TEXT/UTF-8_"$compteur_tableau-$compteur".txt | tr -s "\n" | egrep -v "^$" > index2.txt;
							tail -n +2 index2.txt > index3.txt;
							paste index2.txt index3.txt | sort | uniq -c | sort -r -n -s -k 1,1 -r > ./DUMP-TEXT/"bigrammes_$compteur_tableau-$compteur".txt;
							# Ecriture dans le tableau de chaque colonne pour chaque ligne dotées de liens cliquables permettant l'accès facile à nos données à partir du tableau crée
							echo "<tr>
											<td>Lien $compteur</td>
											<td><span sty<a target=\_blank\" href=\"$ligne\">$ligne</a></td>
											<td>$codeHttp</td>
											<td><a href=\"../PAGES-ASPIREES/$compteur_tableau-$compteur.html\">PA $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/UTF-8_$compteur_tableau-$compteur.txt\">DUMP $compteur</a></td>
											<td>$encodageURL</td>
											<td>$compteurMotif</td>
											<td><a href=\"../CONTEXTES/$compteur_tableau-$compteur.txt\"> CONTEXTE TXT $compteur</a></td>
											<td><a href=\"../CONTEXTES/url_$compteur_tableau-$compteur.html\" > CONTEXTE HTML $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/index_$compteur_tableau-$compteur.txt\"> INDEX $compteur</a></td>
											<td><a href=\"../DUMP-TEXT/bigrammes_$compteur_tableau-$compteur.txt\">Bigrammes $compteur</a></td></tr></tr>" >> $montableau;
											
					fi	# Fermeture de if  [[ $encodageURL == "UTF-8" ]] 
					
				# SINON ( si le code http n'est pas 200 )
				else
							# On le signale sur la sortie standard à l'utilisateur
							echo "---Problème HTTP---";
							# Et on le signale dans le tableau par des ' ?? '
							echo "<tr><td>$compteur</td><td><a target=\"_blank\" href=\"$line\">$line</a></td><td>$codeHttp</td><td> - </td><td> - </td><td>??</td><td>??</td><td>??</td><td>??</td><td>??</td><td>??</td></tr>" >> $montableau;
						
				fi	# Fermeture de if [[ $codeHttp == 200 ]]		
				
				# Incrémentation du compteur d'URL ( vaut 1 pour la première URL traitée , 2 pour la deuxième URL traitée et ainsi de suite pour autant de lignes qu'il y' a d'URL indiquée dans chaque fichier texte d'URL )
				compteur=$((compteur + 1));
			# Indication au programme du chemin vers les fichiers d'URL qu'il doit traiter
			# ( pour chaque ligne d'un fichier texte d'URLS )
			done < $dossierURL/$fichier
		# Fermeture de la table propre à chaque langue ( chaque fichier texte contenu dans le répertoire URLS
		echo "</table>" >> $montableau;
		# Un ( tout petit ) peu de style 
		echo "<p><hr color=\"blue\"></p>" >> $montableau;
		# Incrémentation de la variable compteur_tableau ( vaut 1 pour le premier tableau , 2 pour le deuxième tableau et ainsi de suite pour autant de fichiers texte d'URLs dans le répertoire 'URL' )
	compteur_tableau=$((compteur_tableau + 1));
	# Fermeture de la boucle traitant chaque fichier texte contenant les URLS dans le répertoire 'URL'
	done
	
# Fermeture du fichier html 
echo "</body></html>" >> $montableau;

# Ecraser les fichiers "index" dans notre repertoire courant qui nous on permi de faire les bigrammes
rm ./index*.txt

### FIN DU SCRIPT ###